"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/build.ts
var build_exports = {};
__export(build_exports, {
  build: () => build,
  prepareDummyIndexHtml: () => prepareDummyIndexHtml,
  prepareEntryClientProxy: () => prepareEntryClientProxy
});
module.exports = __toCommonJS(build_exports);
var import_vite = __toESM(require("vite"));
var import_promises2 = __toESM(require("fs/promises"));
var import_path2 = __toESM(require("path"));

// src/util/config.ts
var import_promises = __toESM(require("fs/promises"));
var import_fs = __toESM(require("fs"));
var import_path = __toESM(require("path"));
var resolveDefaultConfig = () => ({
  clientEntry: "src/entry-client.tsx",
  ssrEntry: "src/entry-ssr.tsx",
  dist: "dist",
  configFilePath: "./vr18s.json",
  metadataFolderName: "__vr18s__"
});
async function loadConfigFile(path3) {
  const defaultConfig = resolveDefaultConfig();
  const configFilePath = path3 || defaultConfig.configFilePath;
  if (import_fs.default.existsSync(configFilePath)) {
    const config = JSON.parse(
      await import_promises.default.readFile(configFilePath, "utf8")
    );
    return {
      clientEntry: config.clientEntry || defaultConfig.clientEntry,
      ssrEntry: config.ssrEntry || defaultConfig.ssrEntry,
      dist: config.dist || defaultConfig.dist,
      metadataFolderName: config.metadataFolderName || defaultConfig.metadataFolderName,
      configFilePath
    };
  } else {
    return defaultConfig;
  }
}
function resolveMetadataFolderPath(config) {
  return import_path.default.join(config.dist, config.metadataFolderName);
}

// src/util/dummy-html.ts
function createDummyIndexHtml(clientEntry) {
  return `<!DOCTYPE html>
    <html>
      <head></head>
      <body>
        <script type="module" src="${clientEntry}"></script>
      </body>
    </html>
    `;
}

// src/build.ts
async function build(options = {}) {
  const config = await loadConfigFile(options.configFilePath);
  const distFolder = config.dist;
  const metadataFolder = resolveMetadataFolderPath(config);
  await import_promises2.default.mkdir(metadataFolder, {
    recursive: true
  });
  await import_promises2.default.writeFile(
    import_path2.default.join(metadataFolder, "vr18s.json"),
    JSON.stringify(config),
    "utf8"
  );
  const tmpFolder = `${__dirname}/tmp`;
  await import_promises2.default.rm(tmpFolder, {
    recursive: true,
    force: true
  });
  await import_promises2.default.mkdir(tmpFolder, {
    recursive: true
  });
  await Promise.all([
    prepareDummyIndexHtml(config, tmpFolder),
    prepareEntryClientProxy(config, tmpFolder)
  ]);
  Promise.all([
    import_vite.default.build({
      build: {
        outDir: `${distFolder}/ssr`,
        ssr: config.ssrEntry
      }
    }),
    import_vite.default.build({
      build: {
        outDir: `${distFolder}/client`
      }
    }),
    import_vite.default.build({
      build: {
        outDir: import_path2.default.join(import_path2.default.relative(tmpFolder, "."), metadataFolder)
      },
      root: tmpFolder
    })
  ]);
}
async function prepareDummyIndexHtml(config, tmpFolder) {
  const dummyIndexHtml = createDummyIndexHtml(
    import_path2.default.basename(config.clientEntry)
  );
  return import_promises2.default.writeFile(import_path2.default.join(tmpFolder, "index.html"), dummyIndexHtml);
}
async function prepareEntryClientProxy(config, tmpFolder) {
  const userCwd = process.cwd();
  return import_promises2.default.writeFile(
    import_path2.default.join(tmpFolder, import_path2.default.basename(config.clientEntry)),
    `import "${import_path2.default.join(
      import_path2.default.relative(tmpFolder, userCwd),
      config.clientEntry
    )}"`
  );
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  build,
  prepareDummyIndexHtml,
  prepareEntryClientProxy
});
