// src/build.ts
import vite from "vite";
import fs2 from "fs/promises";
import path2 from "path";

// src/util/config.ts
import fs from "fs/promises";
import fsSync from "fs";
import path from "path";
var resolveDefaultConfig = () => ({
  clientEntry: "src/entry-client.tsx",
  ssrEntry: "src/entry-ssr.tsx",
  dist: "dist",
  configFilePath: "./vr18s.json",
  metadataFolderName: "__vr18s__"
});
async function loadConfigFile(path3) {
  const defaultConfig = resolveDefaultConfig();
  const configFilePath = path3 || defaultConfig.configFilePath;
  if (fsSync.existsSync(configFilePath)) {
    const config = JSON.parse(
      await fs.readFile(configFilePath, "utf8")
    );
    return {
      clientEntry: config.clientEntry || defaultConfig.clientEntry,
      ssrEntry: config.ssrEntry || defaultConfig.ssrEntry,
      dist: config.dist || defaultConfig.dist,
      metadataFolderName: config.metadataFolderName || defaultConfig.metadataFolderName,
      configFilePath
    };
  } else {
    return defaultConfig;
  }
}
function resolveMetadataFolderPath(config) {
  return path.join(config.dist, config.metadataFolderName);
}

// src/util/dummy-html.ts
function createDummyIndexHtml(clientEntry) {
  return `<!DOCTYPE html>
    <html>
      <head></head>
      <body>
        <script type="module" src="${clientEntry}"></script>
      </body>
    </html>
    `;
}

// src/build.ts
async function build(options = {}) {
  const config = await loadConfigFile(options.configFilePath);
  const distFolder = config.dist;
  const metadataFolder = resolveMetadataFolderPath(config);
  await fs2.mkdir(metadataFolder, {
    recursive: true
  });
  await fs2.writeFile(
    path2.join(metadataFolder, "vr18s.json"),
    JSON.stringify(config),
    "utf8"
  );
  const tmpFolder = `${__dirname}/tmp`;
  await fs2.rm(tmpFolder, {
    recursive: true,
    force: true
  });
  await fs2.mkdir(tmpFolder, {
    recursive: true
  });
  await Promise.all([
    prepareDummyIndexHtml(config, tmpFolder),
    prepareEntryClientProxy(config, tmpFolder)
  ]);
  Promise.all([
    vite.build({
      build: {
        outDir: `${distFolder}/ssr`,
        ssr: config.ssrEntry
      }
    }),
    vite.build({
      build: {
        outDir: `${distFolder}/client`
      }
    }),
    vite.build({
      build: {
        outDir: path2.join(path2.relative(tmpFolder, "."), metadataFolder)
      },
      root: tmpFolder
    })
  ]);
}
async function prepareDummyIndexHtml(config, tmpFolder) {
  const dummyIndexHtml = createDummyIndexHtml(
    path2.basename(config.clientEntry)
  );
  return fs2.writeFile(path2.join(tmpFolder, "index.html"), dummyIndexHtml);
}
async function prepareEntryClientProxy(config, tmpFolder) {
  const userCwd = process.cwd();
  return fs2.writeFile(
    path2.join(tmpFolder, path2.basename(config.clientEntry)),
    `import "${path2.join(
      path2.relative(tmpFolder, userCwd),
      config.clientEntry
    )}"`
  );
}
export {
  build,
  prepareDummyIndexHtml,
  prepareEntryClientProxy
};
